@@ -28,6 +28,7 @@
 #include "gtklabel.h"
 #include "gtkmarshalers.h"
 #include "gtkradiobutton.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkalias.h"
 
@@ -113,7 +114,7 @@ gtk_radio_button_class_init (GtkRadioButtonClass *class)
 							P_("Group"),
 							P_("The radio button whose group this widget belongs to."),
 							GTK_TYPE_RADIO_BUTTON,
-							G_PARAM_WRITABLE));
+							GTK_PARAM_WRITABLE));
   object_class->destroy = gtk_radio_button_destroy;
 
   widget_class->focus = gtk_radio_button_focus;
@@ -125,7 +126,7 @@ gtk_radio_button_class_init (GtkRadioButtonClass *class)
   class->group_changed = NULL;
 
   /**
-   * GtkStyle::group-changed:
+   * GtkRadioButton::group-changed:
    * @style: the object which received the signal
    *
    * Emitted when the group of radio buttons that a radio button belongs
@@ -257,6 +258,7 @@ gtk_radio_button_set_group (GtkRadioButton *radio_button,
 
   g_object_ref (radio_button);
   
+  g_object_notify (G_OBJECT (radio_button), "group");
   g_signal_emit (radio_button, group_changed_signal, 0);
   if (old_group_singleton)
     {
@@ -320,7 +322,10 @@ gtk_radio_button_new_with_mnemonic (GSList      *group,
 {
   GtkWidget *radio_button;
 
-  radio_button = g_object_new (GTK_TYPE_RADIO_BUTTON, "label", label, "use_underline", TRUE, NULL);
+  radio_button = g_object_new (GTK_TYPE_RADIO_BUTTON, 
+			       "label", label, 
+			       "use-underline", TRUE, 
+			       NULL);
 
   if (group)
     gtk_radio_button_set_group (GTK_RADIO_BUTTON (radio_button), group);
@@ -681,7 +686,7 @@ gtk_radio_button_draw_indicator (GtkCheckButton *check_button,
       toggle_button = GTK_TOGGLE_BUTTON (check_button);
 
       gtk_widget_style_get (widget,
-			    "interior_focus", &interior_focus,
+			    "interior-focus", &interior_focus,
 			    "focus-line-width", &focus_width,
 			    "focus-padding", &focus_pad,
 			    NULL);
